<?php
// Тестирование дедупликации для Sypex Backuper
header('Content-type: text/plain; charset=UTF-8');
set_time_limit(300);

$dir_to_backup = './';
$archive_file  = '../test.sxb';

$t = microtime(1);

$sxb = new SxB($archive_file);
$sxb->addDirBlock($dir_to_backup);
$sxb->close();

$sxb->info['time'] = round(microtime(1) - $t, 5);

echo "<source lang=\"php\">\n";
foreach($sxb->info AS $k => $v){
	if(in_array($k, array('full_size', 'dup_size', 'bf_size', 'bf_dup_size', 'archive_size'))) $v = round($v / (1024*1024), 3);
	printf("%-12s %s\n", $k, $v);
}
echo "</source>\n";

class SxB {
	protected $fh;
	protected $blockSize = 32768;
	protected $file = '';
	public $catalog = array();
	public $info = array(
		'dirs'          => 0,
		'files'         => 0,
		'full_size'     => 0,
		'dup_size'      => 0,
		'archive_size'  => 0,
		'duplicates'    => 0,
		'bigfiles'      => 0,
		'bf_blocks'     => 0,
		'bf_dup'        => 0,
		'bf_size'       => 0,
		'bf_dup_size'   => 0
	);

	public function __construct($file_name, $compression = 3){
		$this->file = $file_name;
		$this->fh = gzopen($file_name, 'w' . $compression);
	}

	public function addDirBlock($dir, $deep = 0){
		if ($dh = opendir($dir)) {
			$dots_found = 2; // Для фильтрации текущего и родительского каталога
			while (($file = readdir($dh)) !== false) {
				if($dots_found && ($file === '.' || $file === '..')) {$dots_found--;continue;} // Ускоренние фильтрации каталогов . и ..

				$full_file = $dir . $file;
				$stat = stat($full_file);
				$this->info['full_size'] += $stat['size'];
				if(is_dir($full_file)){
					$this->info['dirs']++;
					$this->addDirBlock($full_file . '/', $deep++);
				}
				else{
					$this->info['files']++;
					// Файлы которые меньше размера блока и больше размера блока обрабатываем по разному
					if ($stat['size'] > $this->blockSize) { // Если больше размера блока, читаем файл по блокам
						$fn = fopen($full_file, 'rb');
						$f_ctx = hash_init('md5'); // md5 файла (для оценки скорости)
						$this->info['bigfiles']++;
						$this->info['bf_size'] += $stat['size'];
						while (($content = fread($fn, $this->blockSize)) != false) {
							$this->info['bf_blocks']++;
							$crc = hash('md5', $content, true); // md5 блока
							hash_update($f_ctx, $content);
							$c_size =  strlen($content);

							if (!isset($this->catalog[$crc . $c_size])) { // Если нет такого блока, то сохраняем
								fwrite($this->fh, $content);
								$this->catalog[$crc . $c_size] = true;
							}
							else {
								$this->info['dup_size'] += $c_size;
								$this->info['duplicates']++;
								$this->info['bf_dup']++;
								$this->info['bf_dup_size'] += $c_size;
							}
						}
						fclose($fn);
						$f_crc = hash_final($f_ctx, true);

					}
					else { // Файл меньше размера блока
						$content = file_get_contents($full_file);
						$crc = hash('md5', $content, true);

						if (!isset($this->catalog[$crc . $stat['size']])) { // Если нет такого блока, то сохраняем
							fwrite($this->fh, $content);
							$this->catalog[$crc . $stat['size']] = true;
						}
						else {
							$this->info['dup_size'] += $stat['size'];
							$this->info['duplicates']++;
						}

					}
				}
			}
		}
	}
	public function close(){
		fclose($this->fh);
		$this->info['archive_size'] = filesize($this->file);
	}
}